# Copyright 2019 The gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import contextlib
import datetime
import subprocess
import sys
import threading
import time

import grpc
import grpc.experimental

_PORT = 5741
_MESSAGE_SIZE = 4
_RESPONSE_COUNT = 32 * 1024

_SERVER_CODE = (
    """
import datetime
import threading
import grpc
from concurrent import futures
from src.python.grpcio_tests.tests.stress import unary_stream_benchmark_pb2
from src.python.grpcio_tests.tests.stress import unary_stream_benchmark_pb2_grpc

class Handler(unary_stream_benchmark_pb2_grpc.UnaryStreamBenchmarkServiceServicer):

  def Benchmark(self, request, context):
    payload = b'\\x00\\x01' * int(request.message_size / 2)
    for _ in range(request.response_count):
      yield unary_stream_benchmark_pb2.BenchmarkResponse(response=payload)


server = grpc.server(futures.ThreadPoolExecutor(max_workers=1))
server.add_insecure_port('[::]:%d')
unary_stream_benchmark_pb2_grpc.add_UnaryStreamBenchmarkServiceServicer_to_server(Handler(), server)
server.start()
server.wait_for_termination()
"""
    % _PORT
)

try:
    from src.python.grpcio_tests.tests.stress import (
        unary_stream_benchmark_pb2_grpc,
    )
    from src.python.grpcio_tests.tests.stress import unary_stream_benchmark_pb2

    _GRPC_CHANNEL_OPTIONS = [
        ("grpc.max_metadata_size", 16 * 1024 * 1024),
        ("grpc.max_receive_message_length", 64 * 1024 * 1024),
        (grpc.experimental.ChannelOptions.SingleThreadedUnaryStream, 1),
    ]

    @contextlib.contextmanager
    def _running_server():
        server_process = subprocess.Popen(
            [sys.executable, "-c", _SERVER_CODE],
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE,
        )
        try:
            yield
        finally:
            server_process.terminate()
            server_process.wait()
            sys.stdout.write("stdout: {}".format(server_process.stdout.read()))
            sys.stdout.flush()
            sys.stdout.write("stderr: {}".format(server_process.stderr.read()))
            sys.stdout.flush()

    def profile(message_size, response_count):
        request = unary_stream_benchmark_pb2.BenchmarkRequest(
            message_size=message_size, response_count=response_count
        )
        with grpc.insecure_channel(
            "[::]:{}".format(_PORT), options=_GRPC_CHANNEL_OPTIONS
        ) as channel:
            stub = (
                unary_stream_benchmark_pb2_grpc.UnaryStreamBenchmarkServiceStub(
                    channel
                )
            )
            start = datetime.datetime.now()
            call = stub.Benchmark(request, wait_for_ready=True)
            for message in call:
                pass
            end = datetime.datetime.now()
        return end - start

    def main():
        with _running_server():
            for i in range(1000):
                latency = profile(_MESSAGE_SIZE, 1024)
                sys.stdout.write("{}\n".format(latency.total_seconds()))
                sys.stdout.flush()

    if __name__ == "__main__":
        main()

except ImportError:
    # NOTE(rbellevi): The test runner should not load this module.
    pass



from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Optional as _Optional

DESCRIPTOR: _descriptor.FileDescriptor

class HelloReply(_message.Message):
    __slots__ = ["message"]
    MESSAGE_FIELD_NUMBER: _ClassVar[int]
    message: str
    def __init__(self, message: _Optional[str] = ...) -> None: ...

class HelloRequest(_message.Message):
    __slots__ = ["name", "num_greetings"]
    NAME_FIELD_NUMBER: _ClassVar[int]
    NUM_GREETINGS_FIELD_NUMBER: _ClassVar[int]
    name: str
    num_greetings: str
    def __init__(self, name: _Optional[str] = ..., num_greetings: _Optional[str] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: contrib/envoy/extensions/filters/http/peak_ewma/v3alpha/peak_ewma.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'contrib/envoy/extensions/filters/http/peak_ewma/v3alpha/peak_ewma.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from xds.annotations.v3 import status_pb2 as xds_dot_annotations_dot_v3_dot_status__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nGcontrib/envoy/extensions/filters/http/peak_ewma/v3alpha/peak_ewma.proto\x12/envoy.extensions.filters.http.peak_ewma.v3alpha\x1a\x1fxds/annotations/v3/status.proto\x1a\x1dudpa/annotations/status.proto\"\x1a\n\x0ePeakEwmaConfig:\x08\xd2\xc6\xa4\xe1\x06\x02\x08\x01\x42\xc0\x01\n=io.envoyproxy.envoy.extensions.filters.http.peak_ewma.v3alphaB\rPeakEwmaProtoP\x01Z^github.com/envoyproxy/go-control-plane/contrib/envoy/extensions/filters/http/peak_ewma/v3alpha\xba\x80\xc8\xd1\x06\x02\x10\x02\xd2\xc6\xa4\xe1\x06\x02\x08\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'contrib.envoy.extensions.filters.http.peak_ewma.v3alpha.peak_ewma_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n=io.envoyproxy.envoy.extensions.filters.http.peak_ewma.v3alphaB\rPeakEwmaProtoP\001Z^github.com/envoyproxy/go-control-plane/contrib/envoy/extensions/filters/http/peak_ewma/v3alpha\272\200\310\321\006\002\020\002\322\306\244\341\006\002\010\001'
  _globals['_PEAKEWMACONFIG']._loaded_options = None
  _globals['_PEAKEWMACONFIG']._serialized_options = b'\322\306\244\341\006\002\010\001'
  _globals['_PEAKEWMACONFIG']._serialized_start=188
  _globals['_PEAKEWMACONFIG']._serialized_end=214
# @@protoc_insertion_point(module_scope)



#!/usr/bin/env python3
# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import os
import signal
import subprocess
import sys
import tempfile
import time

import yaml

argp = argparse.ArgumentParser(
    description="Runs a DNS server for LB interop tests"
)
argp.add_argument(
    "-l",
    "--grpclb_ips",
    default=None,
    type=str,
    help="Comma-separated list of IP addresses of balancers",
)
argp.add_argument(
    "-f",
    "--fallback_ips",
    default=None,
    type=str,
    help="Comma-separated list of IP addresses of fallback servers",
)
argp.add_argument(
    "-c",
    "--cause_no_error_no_data_for_balancer_a_record",
    default=False,
    action="store_const",
    const=True,
    help=(
        "Used for testing the case in which the grpclb "
        "balancer A record lookup results in a DNS NOERROR response "
        "but with no ANSWER section i.e. no addresses"
    ),
)
args = argp.parse_args()

balancer_records = []
grpclb_ips = args.grpclb_ips.split(",")
if grpclb_ips[0]:
    for ip in grpclb_ips:
        balancer_records.append(
            {
                "TTL": "2100",
                "data": ip,
                "type": "A",
            }
        )
fallback_records = []
fallback_ips = args.fallback_ips.split(",")
if fallback_ips[0]:
    for ip in fallback_ips:
        fallback_records.append(
            {
                "TTL": "2100",
                "data": ip,
                "type": "A",
            }
        )
records_config_yaml = {
    "resolver_tests_common_zone_name": "test.google.fr.",
    "resolver_component_tests": [
        {
            "records": {
                "_grpclb._tcp.server": [
                    {
                        "TTL": "2100",
                        "data": "0 0 12000 balancer",
                        "type": "SRV",
                    },
                ],
                "balancer": balancer_records,
                "server": fallback_records,
            }
        }
    ],
}
if args.cause_no_error_no_data_for_balancer_a_record:
    balancer_records = records_config_yaml["resolver_component_tests"][0][
        "records"
    ]["balancer"]
    assert not balancer_records
    # Insert a TXT record at the balancer.test.google.fr. domain.
    # This TXT record won't actually be resolved or used by gRPC clients;
    # inserting this record is just a way get the balancer.test.google.fr.
    # A record queries to return NOERROR DNS responses that also have no
    # ANSWER section, in order to simulate this failure case.
    balancer_records.append(
        {
            "TTL": "2100",
            "data": "arbitrary string that wont actually be resolved",
            "type": "TXT",
        }
    )
# Generate the actual DNS server records config file
records_config_path = tempfile.mktemp()
with open(records_config_path, "w") as records_config_generated:
    records_config_generated.write(yaml.dump(records_config_yaml))

with open(records_config_path, "r") as records_config_generated:
    sys.stderr.write("===== DNS server records config: =====\n")
    sys.stderr.write(records_config_generated.read())
    sys.stderr.write("======================================\n")

# Run the DNS server
# Note that we need to add the extra
# A record for metadata.google.internal in order for compute engine
# OAuth creds and ALTS creds to work.
# TODO(apolcyn): should metadata.google.internal always resolve
# to 169.254.169.254?
subprocess.check_output(
    [
        "/var/local/git/grpc/test/cpp/naming/utils/dns_server.py",
        "--port=53",
        "--records_config_path",
        records_config_path,
        "--add_a_record=metadata.google.internal:169.254.169.254",
    ]
)



# Copyright 2020 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import asyncio
import logging

from grpc.experimental import aio

from src.proto.grpc.testing import worker_service_pb2_grpc
from tests_aio.benchmark import worker_servicer


async def run_worker_server(port: int) -> None:
    server = aio.server()

    servicer = worker_servicer.WorkerServicer()
    worker_service_pb2_grpc.add_WorkerServiceServicer_to_server(
        servicer, server
    )

    server.add_insecure_port("[::]:{}".format(port))

    await server.start()

    await servicer.wait_for_quit()
    await server.stop(None)


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    parser = argparse.ArgumentParser(
        description="gRPC Python performance testing worker"
    )
    parser.add_argument(
        "--driver_port",
        type=int,
        dest="port",
        help="The port the worker should listen on",
    )
    parser.add_argument(
        "--uvloop", action="store_true", help="Use uvloop or not"
    )
    args = parser.parse_args()

    if args.uvloop:
        import uvloop

        asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())
        loop = uvloop.new_event_loop()
        asyncio.set_event_loop(loop)

    asyncio.get_event_loop().run_until_complete(run_worker_server(args.port))



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/filters/http/proto_api_scrubber/v3/matcher_actions.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/filters/http/proto_api_scrubber/v3/matcher_actions.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from xds.annotations.v3 import status_pb2 as xds_dot_annotations_dot_v3_dot_status__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nIenvoy/extensions/filters/http/proto_api_scrubber/v3/matcher_actions.proto\x12\x33\x65nvoy.extensions.filters.http.proto_api_scrubber.v3\x1a\x1fxds/annotations/v3/status.proto\x1a\x1dudpa/annotations/status.proto\"\x13\n\x11RemoveFieldActionB\xdb\x01\nAio.envoyproxy.envoy.extensions.filters.http.proto_api_scrubber.v3B\x13MatcherActionsProtoP\x01Zogithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/proto_api_scrubber/v3;proto_api_scrubberv3\xba\x80\xc8\xd1\x06\x02\x10\x02\xd2\xc6\xa4\xe1\x06\x02\x08\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.filters.http.proto_api_scrubber.v3.matcher_actions_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\nAio.envoyproxy.envoy.extensions.filters.http.proto_api_scrubber.v3B\023MatcherActionsProtoP\001Zogithub.com/envoyproxy/go-control-plane/envoy/extensions/filters/http/proto_api_scrubber/v3;proto_api_scrubberv3\272\200\310\321\006\002\020\002\322\306\244\341\006\002\010\001'
  _globals['_REMOVEFIELDACTION']._serialized_start=194
  _globals['_REMOVEFIELDACTION']._serialized_end=213
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/config/rbac/v2/rbac.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/config/rbac/v2/rbac.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.api.v2.core import address_pb2 as envoy_dot_api_dot_v2_dot_core_dot_address__pb2
from envoy.api.v2.route import route_components_pb2 as envoy_dot_api_dot_v2_dot_route_dot_route__components__pb2
from envoy.type.matcher import metadata_pb2 as envoy_dot_type_dot_matcher_dot_metadata__pb2
from envoy.type.matcher import path_pb2 as envoy_dot_type_dot_matcher_dot_path__pb2
from envoy.type.matcher import string_pb2 as envoy_dot_type_dot_matcher_dot_string__pb2
from google.api.expr.v1alpha1 import syntax_pb2 as google_dot_api_dot_expr_dot_v1alpha1_dot_syntax__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1f\x65nvoy/config/rbac/v2/rbac.proto\x12\x14\x65nvoy.config.rbac.v2\x1a\x1f\x65nvoy/api/v2/core/address.proto\x1a)envoy/api/v2/route/route_components.proto\x1a!envoy/type/matcher/metadata.proto\x1a\x1d\x65nvoy/type/matcher/path.proto\x1a\x1f\x65nvoy/type/matcher/string.proto\x1a%google/api/expr/v1alpha1/syntax.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xe3\x01\n\x04RBAC\x12\x31\n\x06\x61\x63tion\x18\x01 \x01(\x0e\x32!.envoy.config.rbac.v2.RBAC.Action\x12:\n\x08policies\x18\x02 \x03(\x0b\x32(.envoy.config.rbac.v2.RBAC.PoliciesEntry\x1aM\n\rPoliciesEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12+\n\x05value\x18\x02 \x01(\x0b\x32\x1c.envoy.config.rbac.v2.Policy:\x02\x38\x01\"\x1d\n\x06\x41\x63tion\x12\t\n\x05\x41LLOW\x10\x00\x12\x08\n\x04\x44\x45NY\x10\x01\"\xbb\x01\n\x06Policy\x12?\n\x0bpermissions\x18\x01 \x03(\x0b\x32 .envoy.config.rbac.v2.PermissionB\x08\xfa\x42\x05\x92\x01\x02\x08\x01\x12=\n\nprincipals\x18\x02 \x03(\x0b\x32\x1f.envoy.config.rbac.v2.PrincipalB\x08\xfa\x42\x05\x92\x01\x02\x08\x01\x12\x31\n\tcondition\x18\x03 \x01(\x0b\x32\x1e.google.api.expr.v1alpha1.Expr\"\xe4\x04\n\nPermission\x12\x39\n\tand_rules\x18\x01 \x01(\x0b\x32$.envoy.config.rbac.v2.Permission.SetH\x00\x12\x38\n\x08or_rules\x18\x02 \x01(\x0b\x32$.envoy.config.rbac.v2.Permission.SetH\x00\x12\x16\n\x03\x61ny\x18\x03 \x01(\x08\x42\x07\xfa\x42\x04j\x02\x08\x01H\x00\x12\x33\n\x06header\x18\x04 \x01(\x0b\x32!.envoy.api.v2.route.HeaderMatcherH\x00\x12\x33\n\x08url_path\x18\n \x01(\x0b\x32\x1f.envoy.type.matcher.PathMatcherH\x00\x12\x36\n\x0e\x64\x65stination_ip\x18\x05 \x01(\x0b\x32\x1c.envoy.api.v2.core.CidrRangeH\x00\x12%\n\x10\x64\x65stination_port\x18\x06 \x01(\rB\t\xfa\x42\x06*\x04\x18\xff\xff\x03H\x00\x12\x37\n\x08metadata\x18\x07 \x01(\x0b\x32#.envoy.type.matcher.MetadataMatcherH\x00\x12\x34\n\x08not_rule\x18\x08 \x01(\x0b\x32 .envoy.config.rbac.v2.PermissionH\x00\x12\x42\n\x15requested_server_name\x18\t \x01(\x0b\x32!.envoy.type.matcher.StringMatcherH\x00\x1a@\n\x03Set\x12\x39\n\x05rules\x18\x01 \x03(\x0b\x32 .envoy.config.rbac.v2.PermissionB\x08\xfa\x42\x05\x92\x01\x02\x08\x01\x42\x0b\n\x04rule\x12\x03\xf8\x42\x01\"\xf8\x05\n\tPrincipal\x12\x36\n\x07\x61nd_ids\x18\x01 \x01(\x0b\x32#.envoy.config.rbac.v2.Principal.SetH\x00\x12\x35\n\x06or_ids\x18\x02 \x01(\x0b\x32#.envoy.config.rbac.v2.Principal.SetH\x00\x12\x16\n\x03\x61ny\x18\x03 \x01(\x08\x42\x07\xfa\x42\x04j\x02\x08\x01H\x00\x12\x46\n\rauthenticated\x18\x04 \x01(\x0b\x32-.envoy.config.rbac.v2.Principal.AuthenticatedH\x00\x12\x35\n\tsource_ip\x18\x05 \x01(\x0b\x32\x1c.envoy.api.v2.core.CidrRangeB\x02\x18\x01H\x00\x12\x38\n\x10\x64irect_remote_ip\x18\n \x01(\x0b\x32\x1c.envoy.api.v2.core.CidrRangeH\x00\x12\x31\n\tremote_ip\x18\x0b \x01(\x0b\x32\x1c.envoy.api.v2.core.CidrRangeH\x00\x12\x33\n\x06header\x18\x06 \x01(\x0b\x32!.envoy.api.v2.route.HeaderMatcherH\x00\x12\x33\n\x08url_path\x18\t \x01(\x0b\x32\x1f.envoy.type.matcher.PathMatcherH\x00\x12\x37\n\x08metadata\x18\x07 \x01(\x0b\x32#.envoy.type.matcher.MetadataMatcherH\x00\x12\x31\n\x06not_id\x18\x08 \x01(\x0b\x32\x1f.envoy.config.rbac.v2.PrincipalH\x00\x1a=\n\x03Set\x12\x36\n\x03ids\x18\x01 \x03(\x0b\x32\x1f.envoy.config.rbac.v2.PrincipalB\x08\xfa\x42\x05\x92\x01\x02\x08\x01\x1aP\n\rAuthenticated\x12\x39\n\x0eprincipal_name\x18\x02 \x01(\x0b\x32!.envoy.type.matcher.StringMatcherJ\x04\x08\x01\x10\x02\x42\x11\n\nidentifier\x12\x03\xf8\x42\x01\x42}\n\"io.envoyproxy.envoy.config.rbac.v2B\tRbacProtoP\x01ZBgithub.com/envoyproxy/go-control-plane/envoy/config/rbac/v2;rbacv2\xba\x80\xc8\xd1\x06\x02\x10\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.config.rbac.v2.rbac_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\"io.envoyproxy.envoy.config.rbac.v2B\tRbacProtoP\001ZBgithub.com/envoyproxy/go-control-plane/envoy/config/rbac/v2;rbacv2\272\200\310\321\006\002\020\001'
  _globals['_RBAC_POLICIESENTRY']._loaded_options = None
  _globals['_RBAC_POLICIESENTRY']._serialized_options = b'8\001'
  _globals['_POLICY'].fields_by_name['permissions']._loaded_options = None
  _globals['_POLICY'].fields_by_name['permissions']._serialized_options = b'\372B\005\222\001\002\010\001'
  _globals['_POLICY'].fields_by_name['principals']._loaded_options = None
  _globals['_POLICY'].fields_by_name['principals']._serialized_options = b'\372B\005\222\001\002\010\001'
  _globals['_PERMISSION_SET'].fields_by_name['rules']._loaded_options = None
  _globals['_PERMISSION_SET'].fields_by_name['rules']._serialized_options = b'\372B\005\222\001\002\010\001'
  _globals['_PERMISSION'].oneofs_by_name['rule']._loaded_options = None
  _globals['_PERMISSION'].oneofs_by_name['rule']._serialized_options = b'\370B\001'
  _globals['_PERMISSION'].fields_by_name['any']._loaded_options = None
  _globals['_PERMISSION'].fields_by_name['any']._serialized_options = b'\372B\004j\002\010\001'
  _globals['_PERMISSION'].fields_by_name['destination_port']._loaded_options = None
  _globals['_PERMISSION'].fields_by_name['destination_port']._serialized_options = b'\372B\006*\004\030\377\377\003'
  _globals['_PRINCIPAL_SET'].fields_by_name['ids']._loaded_options = None
  _globals['_PRINCIPAL_SET'].fields_by_name['ids']._serialized_options = b'\372B\005\222\001\002\010\001'
  _globals['_PRINCIPAL'].oneofs_by_name['identifier']._loaded_options = None
  _globals['_PRINCIPAL'].oneofs_by_name['identifier']._serialized_options = b'\370B\001'
  _globals['_PRINCIPAL'].fields_by_name['any']._loaded_options = None
  _globals['_PRINCIPAL'].fields_by_name['any']._serialized_options = b'\372B\004j\002\010\001'
  _globals['_PRINCIPAL'].fields_by_name['source_ip']._loaded_options = None
  _globals['_PRINCIPAL'].fields_by_name['source_ip']._serialized_options = b'\030\001'
  _globals['_RBAC']._serialized_start=328
  _globals['_RBAC']._serialized_end=555
  _globals['_RBAC_POLICIESENTRY']._serialized_start=447
  _globals['_RBAC_POLICIESENTRY']._serialized_end=524
  _globals['_RBAC_ACTION']._serialized_start=526
  _globals['_RBAC_ACTION']._serialized_end=555
  _globals['_POLICY']._serialized_start=558
  _globals['_POLICY']._serialized_end=745
  _globals['_PERMISSION']._serialized_start=748
  _globals['_PERMISSION']._serialized_end=1360
  _globals['_PERMISSION_SET']._serialized_start=1283
  _globals['_PERMISSION_SET']._serialized_end=1347
  _globals['_PRINCIPAL']._serialized_start=1363
  _globals['_PRINCIPAL']._serialized_end=2123
  _globals['_PRINCIPAL_SET']._serialized_start=1961
  _globals['_PRINCIPAL_SET']._serialized_end=2022
  _globals['_PRINCIPAL_AUTHENTICATED']._serialized_start=2024
  _globals['_PRINCIPAL_AUTHENTICATED']._serialized_end=2104
# @@protoc_insertion_point(module_scope)



# Copyright 2019 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


cdef class _AioCall(GrpcCallWrapper):
    cdef:
        readonly AioChannel _channel
        list _references
        object _deadline
        list _done_callbacks

        # Caches the picked event loop, so we can avoid the 30ns overhead each
        # time we need access to the event loop.
        object _loop

        # Flag indicates whether cancel being called or not. Cancellation from
        # Core or peer works perfectly fine with normal procedure. However, we
        # need this flag to clean up resources for cancellation from the
        # application layer. Directly cancelling tasks might cause segfault
        # because Core is holding a pointer for the callback handler.
        bint _is_locally_cancelled

        # Following attributes are used for storing the status of the call and
        # the initial metadata. Waiters are used for pausing the execution of
        # tasks that are asking for one of the field when they are not yet
        # available.
        readonly AioRpcStatus _status
        readonly tuple _initial_metadata
        list _waiters_status
        list _waiters_initial_metadata

        int _send_initial_metadata_flags
        object _call_tracer_capsule

    cdef void _create_grpc_call(self, object timeout, bytes method, CallCredentials credentials) except *
    cdef void _maybe_set_client_call_tracer_on_call(self, bytes method) except *
    cdef void _set_status(self, AioRpcStatus status) except *
    cdef void _set_initial_metadata(self, tuple initial_metadata) except *



#!/usr/bin/env python3

# Copyright 2023 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import sys

from mako.template import Template

seq_state = Template(
    """
<%def name="decl(promise_name, i, n)">
using Promise${i} = ${promise_name};
% if i < n-1:
using NextFactory${i} = OncePromiseFactory<typename Promise${i}::Result, F${i}>;
${decl(f"typename NextFactory{i}::Promise", i+1, n)}
% endif
</%def>

<%def name="state(i, n)">
% if i == 0:
Promise0 current_promise;
NextFactory0 next_factory;
% elif i == n-1:
union {
    struct { ${state(i-1, n)} } prior;
    Promise${i} current_promise;
};
% else:
union {
    struct { ${state(i-1, n)} } prior;
    P${i} current_promise;
};
NextFactory${i} next_factory;
% endif
</%def>

template <template<typename> class Traits, typename P, ${",".join(f"typename F{i}" for i in range(0,n-1))}>
struct SeqState<Traits, P, ${",".join(f"F{i}" for i in range(0,n-1))}> {
<% name="PromiseLike<P>" %>
% for i in range(0,n-1):
using Promise${i} = ${name};
using PromiseResult${i} = typename Promise${i}::Result;
using PromiseResultTraits${i} = Traits<PromiseResult${i}>;
using NextFactory${i} = OncePromiseFactory<typename PromiseResultTraits${i}::UnwrappedType, F${i}>;
<% name=f"typename NextFactory{i}::Promise" %>\\
% endfor
using Promise${n-1} = ${name};
using PromiseResult${n-1} = typename Promise${n-1}::Result;
using PromiseResultTraits${n-1} = Traits<PromiseResult${n-1}>;
using Result = typename PromiseResultTraits${n-1}::WrappedType;
% if n == 1:
Promise0 current_promise;
% else:
%  for i in range(0,n-1):
struct Running${i} {
%   if i != 0:
union {
  GPR_NO_UNIQUE_ADDRESS Running${i-1} prior;
%   endif
  GPR_NO_UNIQUE_ADDRESS Promise${i} current_promise;
%   if i != 0:
};
%   endif
GPR_NO_UNIQUE_ADDRESS NextFactory${i} next_factory;
};
%  endfor
union {
    GPR_NO_UNIQUE_ADDRESS Running${n-2} prior;
    GPR_NO_UNIQUE_ADDRESS Promise${n-1} current_promise;
};
% endif
  enum class State : uint8_t { ${",".join(f"kState{i}" for i in range(0,n))} };
  GPR_NO_UNIQUE_ADDRESS State state = State::kState0;
  GPR_NO_UNIQUE_ADDRESS DebugLocation whence;

  GPR_ATTRIBUTE_ALWAYS_INLINE_FUNCTION SeqState(P&& p,
           ${",".join(f"F{i}&& f{i}" for i in range(0,n-1))},
           DebugLocation whence) noexcept: whence(whence)  {
    Construct(&${"prior."*(n-1)}current_promise, std::forward<P>(p));
% for i in range(0,n-1):
    Construct(&${"prior."*(n-1-i)}next_factory, std::forward<F${i}>(f${i}));
% endfor
  }
  GPR_ATTRIBUTE_ALWAYS_INLINE_FUNCTION ~SeqState() {
    switch (state) {
% for i in range(0,n-1):
     case State::kState${i}:
      Destruct(&${"prior."*(n-1-i)}current_promise);
      goto tail${i};
% endfor
     case State::kState${n-1}:
      Destruct(&current_promise);
      return;
    }
% for i in range(0,n-1):
tail${i}:
    Destruct(&${"prior."*(n-1-i)}next_factory);
% endfor
  }
  GPR_ATTRIBUTE_ALWAYS_INLINE_FUNCTION SeqState(const SeqState& other) noexcept = delete;
  SeqState& operator=(const SeqState& other) = delete;
  GPR_ATTRIBUTE_ALWAYS_INLINE_FUNCTION SeqState(SeqState&& other) noexcept : state(other.state), whence(other.whence) {
    GRPC_DCHECK(state == State::kState0);
    Construct(&${"prior."*(n-1)}current_promise,
              std::move(other.${"prior."*(n-1)}current_promise));
% for i in range(0,n-1):
    Construct(&${"prior."*(n-1-i)}next_factory,
              std::move(other.${"prior."*(n-1-i)}next_factory));
% endfor
  }
  GPR_ATTRIBUTE_ALWAYS_INLINE_FUNCTION SeqState& operator=(SeqState&& other) = delete;
  void ToProto(grpc_channelz_v2_Promise_CompositionKind kind, grpc_channelz_v2_Promise* promise_proto, upb_Arena* arena) const {
    auto* seq_promise = grpc_channelz_v2_Promise_mutable_seq_promise(promise_proto, arena);
    grpc_channelz_v2_Promise_Seq_set_kind(seq_promise, kind);
    auto** steps = grpc_channelz_v2_Promise_Seq_resize_steps(seq_promise, ${n}, arena);
    for (int i = 0; i < ${n}; i++) {
        steps[i] = grpc_channelz_v2_Promise_SeqStep_new(arena);
    }
    grpc_channelz_v2_Promise_SeqStep_set_factory(steps[0], StdStringToUpbString(TypeName<P>()));
    if (state == State::kState0) {
        PromiseAsProto(${"prior."*(n-1)}current_promise, grpc_channelz_v2_Promise_SeqStep_mutable_polling_promise(steps[0], arena), arena);
    }
% for i in range(1,n):
    grpc_channelz_v2_Promise_SeqStep_set_factory(steps[${i}], StdStringToUpbString(TypeName<F${i-1}>()));
    if (state == State::kState${i}) {
      PromiseAsProto(${"prior."*(n-1-i)}current_promise, grpc_channelz_v2_Promise_SeqStep_mutable_polling_promise(steps[${i}], arena), arena);
    }
% endfor
  }
  GPR_ATTRIBUTE_ALWAYS_INLINE_FUNCTION Poll<Result> PollOnce() {
    switch (state) {
% for i in range(0,n-1):
      case State::kState${i}: {
        GRPC_TRACE_LOG(promise_primitives, INFO).AtLocation(whence.file(), whence.line())
                << "seq[" << this << "]: begin poll step ${i+1}/${n}";
        auto result = ${"prior."*(n-1-i)}current_promise();
        PromiseResult${i}* p = result.value_if_ready();
        GRPC_TRACE_LOG(promise_primitives, INFO).AtLocation(whence.file(), whence.line())
                << "seq[" << this << "]: poll step ${i+1}/${n} gets "
                << (p != nullptr
                    ? (PromiseResultTraits${i}::IsOk(*p)
                      ? "ready"
                      : absl::StrCat("early-error:", PromiseResultTraits${i}::ErrorString(*p)).c_str())
                    : "pending");
        if (p == nullptr) return Pending{};
        if (!PromiseResultTraits${i}::IsOk(*p)) {
          return PromiseResultTraits${i}::template ReturnValue<Result>(std::move(*p));
        }
        Destruct(&${"prior."*(n-1-i)}current_promise);
        auto next_promise = PromiseResultTraits${i}::CallFactory(&${"prior."*(n-1-i)}next_factory, std::move(*p));
        Destruct(&${"prior."*(n-1-i)}next_factory);
        Construct(&${"prior."*(n-2-i)}current_promise, std::move(next_promise));
        state = State::kState${i+1};
      }
      [[fallthrough]];
% endfor
      default:
      case State::kState${n-1}: {
        GRPC_TRACE_LOG(promise_primitives, INFO).AtLocation(whence.file(), whence.line())
                << "seq[" << this << "]: begin poll step ${n}/${n}";
        auto result = current_promise();
        GRPC_TRACE_LOG(promise_primitives, INFO).AtLocation(whence.file(), whence.line())
                << "seq[" << this << "]: poll step ${n}/${n} gets "
                << (result.ready()? "ready" : "pending");
        auto* p = result.value_if_ready();
        if (p == nullptr) return Pending{};
        return Result(std::move(*p));
      }
    }
  }
};"""
)

front_matter = """
#ifndef GRPC_SRC_CORE_LIB_PROMISE_DETAIL_SEQ_STATE_H
#define GRPC_SRC_CORE_LIB_PROMISE_DETAIL_SEQ_STATE_H

// This file is generated by tools/codegen/core/gen_seq.py

#include <grpc/support/port_platform.h>

#include <stdint.h>

#include <utility>

#include "absl/log/log.h"
#include "absl/base/attributes.h"
#include "absl/strings/str_cat.h"

#include "src/core/lib/debug/trace.h"
#include "src/core/util/construct_destruct.h"
#include "src/core/util/debug_location.h"
#include "src/core/lib/promise/detail/promise_factory.h"
#include "src/core/lib/promise/detail/promise_like.h"
#include "src/core/lib/promise/poll.h"
#include "src/core/lib/promise/promise.h"

// A sequence under some traits for some set of callables P, Fs.
// P should be a promise-like object that yields a value.
// Fs... should be promise-factory-like objects that take the value from the
// previous step and yield a promise. Note that most of the machinery in
// PromiseFactory exists to make it possible for those promise-factory-like
// objects to be anything that's convenient.
// Traits defines how we move from one step to the next. Traits sets up the
// wrapping and escape handling for the sequence.
// Promises return wrapped values that the trait can inspect and unwrap before
// passing them to the next element of the sequence. The trait can
// also interpret a wrapped value as an escape value, which terminates
// evaluation of the sequence immediately yielding a result. Traits for type T
// have the members:
//  * type UnwrappedType - the type after removing wrapping from T (i.e. for
//    TrySeq, T=StatusOr<U> yields UnwrappedType=U).
//  * type WrappedType - the type after adding wrapping if it doesn't already
//    exist (i.e. for TrySeq if T is not Status/StatusOr/void, then
//    WrappedType=StatusOr<T>; if T is Status then WrappedType=Status (it's
//    already wrapped!))
//  * template <typename Next> void CallFactory(Next* next_factory, T&& value) -
//    call promise factory next_factory with the result of unwrapping value, and
//    return the resulting promise.
//  * template <typename Result, typename RunNext> Poll<Result>
//    CheckResultAndRunNext(T prior, RunNext run_next) - examine the value of
//    prior, and decide to escape or continue. If escaping, return the final
//    sequence value of type Poll<Result>. If continuing, return the value of
//    run_next(std::move(prior)).
//
// A state contains the current promise, and the promise factory to turn the
// result of the current promise into the next state's promise. We play a shell
// game such that the prior state and our current promise are kept in a union,
// and the next promise factory is kept alongside in the state struct.
// Recursively this guarantees that the next functions get initialized once, and
// destroyed once, and don't need to be moved around in between, which avoids a
// potential O(n**2) loop of next factory moves had we used a variant of states
// here. The very first state does not have a prior state, and so that state has
// a partial specialization below. The final state does not have a next state;
// that state is inlined in BasicSeq since that was simpler to type.

namespace grpc_core {
namespace promise_detail {
template <template<typename> class Traits, typename P, typename... Fs>
struct SeqState;
"""

end_matter = """
}  // namespace promise_detail
}  // namespace grpc_core

#endif  // GRPC_SRC_CORE_LIB_PROMISE_DETAIL_SEQ_STATE_H
"""


# utility: print a big comment block into a set of files
def put_banner(files, banner):
    for f in files:
        for line in banner:
            print("// %s" % line, file=f)
        print("", file=f)


with open(sys.argv[0]) as my_source:
    copyright = []
    for line in my_source:
        if line[0] != "#":
            break
    for line in my_source:
        if line[0] == "#":
            copyright.append(line)
            break
    for line in my_source:
        if line[0] != "#":
            break
        copyright.append(line)

copyright = [line[2:].rstrip() for line in copyright]

output_file = (
    "/".join(sys.argv[0].split("/")[:-4])
    + "/src/core/lib/promise/detail/seq_state.h"
)
print(output_file)

with open(output_file, "w") as f:
    put_banner([f], copyright)
    print(front_matter, file=f)
    for n in range(2, 14):
        print(seq_state.render(n=n), file=f)
    print(end_matter, file=f)



# Copyright 2020 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Testing the channel_ready function."""

import asyncio
import gc
import logging
import socket
import time
import unittest

import grpc
from grpc.experimental import aio

from tests.unit.framework.common import get_socket
from tests.unit.framework.common import test_constants
from tests_aio.unit import _common
from tests_aio.unit._test_base import AioTestBase
from tests_aio.unit._test_server import start_test_server


class TestChannelReady(AioTestBase):
    async def setUp(self):
        address, self._port, self._socket = get_socket(
            listen=False, sock_options=(socket.SO_REUSEADDR,)
        )
        self._channel = aio.insecure_channel(f"{address}:{self._port}")
        self._socket.close()

    async def tearDown(self):
        await self._channel.close()

    async def test_channel_ready_success(self):
        # Start `channel_ready` as another Task
        channel_ready_task = self.loop.create_task(
            self._channel.channel_ready()
        )

        # Wait for TRANSIENT_FAILURE
        await _common.block_until_certain_state(
            self._channel, grpc.ChannelConnectivity.TRANSIENT_FAILURE
        )

        server = None
        try:
            # Start the server
            _, server = await start_test_server(port=self._port)

            # The RPC should recover itself
            await channel_ready_task
        finally:
            if server:
                await server.stop(None)

    @unittest.skip(
        "skipping due to flake: https://github.com/grpc/grpc/issues/37949"
    )
    async def test_channel_ready_blocked(self):
        with self.assertRaises(asyncio.TimeoutError):
            await asyncio.wait_for(
                self._channel.channel_ready(), test_constants.SHORT_TIMEOUT
            )


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    unittest.main(verbosity=2)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/watchdog/v3/abort_action.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/watchdog/v3/abort_action.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n$envoy/watchdog/v3/abort_action.proto\x12\x11\x65nvoy.watchdog.v3\x1a\x1egoogle/protobuf/duration.proto\x1a\x1dudpa/annotations/status.proto\"E\n\x11\x41\x62ortActionConfig\x12\x30\n\rwait_duration\x18\x01 \x01(\x0b\x32\x19.google.protobuf.DurationB\x82\x01\n\x1fio.envoyproxy.envoy.watchdog.v3B\x10\x41\x62ortActionProtoP\x01ZCgithub.com/envoyproxy/go-control-plane/envoy/watchdog/v3;watchdogv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.watchdog.v3.abort_action_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\037io.envoyproxy.envoy.watchdog.v3B\020AbortActionProtoP\001ZCgithub.com/envoyproxy/go-control-plane/envoy/watchdog/v3;watchdogv3\272\200\310\321\006\002\020\002'
  _globals['_ABORTACTIONCONFIG']._serialized_start=122
  _globals['_ABORTACTIONCONFIG']._serialized_end=191
# @@protoc_insertion_point(module_scope)



# Copyright 2016 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests the implementations module of the gRPC Python Beta API."""

import datetime
import unittest

from grpc.beta import implementations
from oauth2client import client as oauth2client_client

from tests.unit import resources


class ChannelCredentialsTest(unittest.TestCase):
    def test_runtime_provided_root_certificates(self):
        channel_credentials = implementations.ssl_channel_credentials()
        self.assertIsInstance(
            channel_credentials, implementations.ChannelCredentials
        )

    def test_application_provided_root_certificates(self):
        channel_credentials = implementations.ssl_channel_credentials(
            resources.test_root_certificates()
        )
        self.assertIsInstance(
            channel_credentials, implementations.ChannelCredentials
        )


class CallCredentialsTest(unittest.TestCase):
    def test_google_call_credentials(self):
        creds = oauth2client_client.GoogleCredentials(
            "token",
            "client_id",
            "secret",
            "refresh_token",
            datetime.datetime(2008, 6, 24),
            "https://refresh.uri.com/",
            "user_agent",
        )
        call_creds = implementations.google_call_credentials(creds)
        self.assertIsInstance(call_creds, implementations.CallCredentials)

    def test_access_token_call_credentials(self):
        call_creds = implementations.access_token_call_credentials("token")
        self.assertIsInstance(call_creds, implementations.CallCredentials)


if __name__ == "__main__":
    unittest.main(verbosity=2)



#!/usr/bin/env python3
# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import print_function

import argparse
import os
import os.path
import shutil
import subprocess
import sys
import tempfile

import grpc_version

parser = argparse.ArgumentParser()
parser.add_argument(
    "--repo-owner", type=str, help="Owner of the GitHub repository to be pushed"
)
parser.add_argument(
    "--doc-branch", type=str, default="python-doc-%s" % grpc_version.VERSION
)
args = parser.parse_args()

SCRIPT_DIR = os.path.dirname(os.path.abspath(__file__))
PROJECT_ROOT = os.path.abspath(os.path.join(SCRIPT_DIR, "..", "..", ".."))

SETUP_PATH = os.path.join(PROJECT_ROOT, "setup.py")
REQUIREMENTS_PATH = os.path.join(PROJECT_ROOT, "requirements.bazel.lock")
DOC_PATH = os.path.join(PROJECT_ROOT, "doc/build")

if "VIRTUAL_ENV" in os.environ:
    VIRTUALENV_DIR = os.environ["VIRTUAL_ENV"]
    PYTHON_PATH = os.path.join(VIRTUALENV_DIR, "bin", "python")
    subprocess_arguments_list = []
else:
    VIRTUALENV_DIR = os.path.join(SCRIPT_DIR, "distrib_virtualenv")
    PYTHON_PATH = os.path.join(VIRTUALENV_DIR, "bin", "python")
    subprocess_arguments_list = [
        ["python3", "-m", "virtualenv", VIRTUALENV_DIR],
    ]

subprocess_arguments_list += [
    [PYTHON_PATH, "-m", "pip", "install", "--upgrade", "pip==19.3.1"],
    [PYTHON_PATH, "-m", "pip", "install", "-r", REQUIREMENTS_PATH],
    [PYTHON_PATH, "-m", "pip", "install", "--upgrade", "Sphinx"],
    [PYTHON_PATH, SETUP_PATH, "doc"],
]

for subprocess_arguments in subprocess_arguments_list:
    print("Running command: {}".format(subprocess_arguments))
    subprocess.check_call(args=subprocess_arguments)

if not args.repo_owner or not args.doc_branch:
    tty_width = int(os.popen("stty size", "r").read().split()[1])
    print("-" * tty_width)
    print("Please check generated Python doc inside doc/build")
    print(
        "To push to a GitHub repo, please provide repo owner and doc branch"
        " name"
    )
else:
    # Create a temporary directory out of tree, checkout gh-pages from the
    # specified repository, edit it, and push it. It's up to the user to then go
    # onto GitHub and make a PR against grpc/grpc:gh-pages.
    repo_parent_dir = tempfile.mkdtemp()
    print("Documentation parent directory: {}".format(repo_parent_dir))
    repo_dir = os.path.join(repo_parent_dir, "grpc")
    python_doc_dir = os.path.join(repo_dir, "python")
    doc_branch = args.doc_branch

    print("Cloning your repository...")
    subprocess.check_call(
        [
            "git",
            "clone",
            "--branch",
            "gh-pages",
            "https://github.com/grpc/grpc",
        ],
        cwd=repo_parent_dir,
    )
    subprocess.check_call(["git", "checkout", "-b", doc_branch], cwd=repo_dir)
    subprocess.check_call(
        [
            "git",
            "remote",
            "add",
            "ssh-origin",
            "git@github.com:%s/grpc.git" % args.repo_owner,
        ],
        cwd=repo_dir,
    )
    print("Updating documentation...")
    shutil.rmtree(python_doc_dir, ignore_errors=True)
    shutil.copytree(DOC_PATH, python_doc_dir)
    print(
        "Attempting to push documentation to %s/%s..."
        % (args.repo_owner, doc_branch)
    )
    try:
        subprocess.check_call(["git", "add", "--all"], cwd=repo_dir)
        subprocess.check_call(
            ["git", "commit", "-m", "Auto-update Python documentation"],
            cwd=repo_dir,
        )
        subprocess.check_call(
            ["git", "push", "--set-upstream", "ssh-origin", doc_branch],
            cwd=repo_dir,
        )
    except subprocess.CalledProcessError:
        print(
            "Failed to push documentation. Examine this directory and push "
            "manually: {}".format(repo_parent_dir)
        )
        sys.exit(1)
    shutil.rmtree(repo_parent_dir)



#!/usr/bin/env python3
# Copyright 2022 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# Fake protobuf compiler for use in the Grpc.Tools MSBuild integration
# unit tests.  Its purpose is to be called from the Grpc.Tools
# Google.Protobuf.Tools.targets MSBuild file instead of the actual protoc
# compiler. This script:
# - parses the command line arguments
# - generates expected dependencies file
# - generates dummy .cs files that are expected by the tests
# - writes a JSON results file containing the arguments passed in

# Configuration is done via environment variables as it is not possible
# to pass additional argument when called from the MSBuild scripts under test.
#
# Environment variables:
# FAKEPROTOC_PROJECTDIR - project directory
# FAKEPROTOC_OUTDIR - output directory for generated files and output file
# FAKEPROTOC_GENERATE_EXPECTED - list of expected generated files in format:
#         file1.proto:csfile1.cs;csfile2.cs|file2.proto:csfile3.cs;csfile4.cs|...

import datetime
import hashlib
import json
import os
import sys

# Set to True to write out debug messages from this script
_dbg = True
# file to which write the debug log
_dbgfile = None


def _open_debug_log(filename):
    """Create debug file for this script."""
    global _dbgfile
    if _dbg:
        # append mode since this script may be called multiple times
        # during one build/test
        _dbgfile = open(filename, "a")


def _close_debug_log():
    """Close the debug log file."""
    if _dbgfile:
        _dbgfile.close()


def _write_debug(msg):
    """Write to the debug log file if debug is enabled."""
    if _dbg and _dbgfile:
        print(msg, file=_dbgfile, flush=True)


def _read_protoc_arguments():
    """
    Get the protoc argument from the command line and
    any response files specified on the command line.

    Returns the list of arguments.
    """
    _write_debug("\nread_protoc_arguments")
    result = []
    for arg in sys.argv[1:]:
        _write_debug("  arg: " + arg)
        if arg.startswith("@"):
            # TODO(jtattermusch): inserting a "commented out" argument feels hacky
            result.append("# RSP file: %s" % arg)
            rsp_file_name = arg[1:]
            result.extend(_read_rsp_file(rsp_file_name))
        else:
            result.append(arg)
    return result


def _read_rsp_file(rspfile):
    """
    Returns list of arguments from a response file.
    """
    _write_debug("\nread_rsp_file: " + rspfile)
    result = []
    with open(rspfile, "r") as rsp:
        for line in rsp:
            line = line.strip()
            _write_debug("    line: " + line)
            result.append(line)
    return result


def _parse_protoc_arguments(protoc_args, projectdir):
    """
    Parse the protoc arguments from the provided list
    """

    _write_debug("\nparse_protoc_arguments")
    arg_dict = {}
    for arg in protoc_args:
        _write_debug("Parsing: %s" % arg)

        # All arguments containing file or directory paths are
        # normalized by converting all '\' and changed to '/'
        if arg.startswith("--"):
            # Assumes that cmdline arguments are always passed in the
            # "--somearg=argvalue", which happens to be the form that
            # msbuild integration uses, but it's not the only way.
            (name, value) = arg.split("=", 1)

            if (
                name == "--dependency_out"
                or name == "--grpc_out"
                or name == "--csharp_out"
            ):
                # For args that contain a path, make the path absolute and normalize it
                # to make it easier to assert equality in tests.
                value = _normalized_absolute_path(value)

            if name == "--proto_path":
                # for simplicity keep this one as relative path rather than absolute path
                # since it is an input file that is always be near the project file
                value = _normalized_relative_to_projectdir(value, projectdir)

            _add_protoc_arg_to_dict(arg_dict, name, value)

        elif arg.startswith("#"):
            pass  # ignore
        else:
            # arg represents a proto file name
            arg = _normalized_relative_to_projectdir(arg, projectdir)
            _add_protoc_arg_to_dict(arg_dict, "protofile", arg)
    return arg_dict


def _add_protoc_arg_to_dict(arg_dict, name, value):
    """
    Add the arguments with name/value to a multi-dictionary of arguments
    """
    if name not in arg_dict:
        arg_dict[name] = []

    arg_dict[name].append(value)


def _normalized_relative_to_projectdir(file, projectdir):
    """Convert a file path to one relative to the project directory."""
    try:
        return _normalize_slashes(
            os.path.relpath(os.path.abspath(file), projectdir)
        )
    except ValueError:
        # On Windows if the paths are on different drives then we get this error
        # Just return the absolute path
        return _normalize_slashes(os.path.abspath(file))


def _normalized_absolute_path(file):
    """Returns normalized absolute path to file."""
    return _normalize_slashes(os.path.abspath(file))


def _normalize_slashes(path):
    """Change all backslashes to forward slashes to make comparing path strings easier."""
    return path.replace("\\", "/")


def _write_or_update_results_json(log_dir, protofile, protoc_arg_dict):
    """Write or update the results JSON file"""

    # Read existing json.
    # Since protoc may be called more than once each build/test if there is
    # more than one protoc file, we read the existing data to add to it.
    fname = os.path.abspath("%s/results.json" % log_dir)
    if os.path.isfile(fname):
        # Load the original contents.
        with open(fname, "r") as forig:
            results_json = json.load(forig)
    else:
        results_json = {}
        results_json["Files"] = {}

    results_json["Files"][protofile] = protoc_arg_dict
    results_json["Metadata"] = {"timestamp": str(datetime.datetime.now())}

    with open(fname, "w") as fout:
        json.dump(results_json, fout, indent=4)


def _parse_generate_expected(generate_expected_str):
    """
    Parse FAKEPROTOC_GENERATE_EXPECTED that specifies the proto files
    and the cs files to generate. We rely on the test to say what is
    expected rather than trying to work it out in this script.

    The format of the input is:
        file1.proto:csfile1.cs;csfile2.cs|file2.proto:csfile3.cs;csfile4.cs|...
    """
    _write_debug("\nparse_generate_expected")

    result = {}
    entries = generate_expected_str.split("|")
    for entry in entries:
        parts = entry.split(":")
        pfile = _normalize_slashes(parts[0])
        csfiles = parts[1].split(";")
        result[pfile] = csfiles
        _write_debug(pfile + " : " + str(csfiles))
    return result


def _get_cs_files_to_generate(protofile, proto_to_generated):
    """Returns list of .cs files to generated based on FAKEPROTOC_GENERATE_EXPECTED env."""
    protoname_normalized = _normalize_slashes(protofile)
    cs_files_to_generate = proto_to_generated.get(protoname_normalized)
    return cs_files_to_generate


def _is_grpc_out_file(csfile):
    """Return true if the file is one that would be generated by gRPC plugin"""
    # This is using the heuristics of checking that the name of the file
    # matches *Grpc.cs which is the name that the gRPC plugin would produce.
    return csfile.endswith("Grpc.cs")


def _generate_cs_files(
    protofile, cs_files_to_generate, grpc_out_dir, csharp_out_dir, projectdir
):
    """Create expected cs files."""
    _write_debug("\ngenerate_cs_files")

    if not cs_files_to_generate:
        _write_debug("No .cs files matching proto file name %s" % protofile)
        return

    if not os.path.isabs(grpc_out_dir):
        # if not absolute, it is relative to project directory
        grpc_out_dir = os.path.abspath("%s/%s" % (projectdir, grpc_out_dir))

    if not os.path.isabs(csharp_out_dir):
        # if not absolute, it is relative to project directory
        csharp_out_dir = os.path.abspath("%s/%s" % (projectdir, csharp_out_dir))

    # Ensure directories exist
    if not os.path.isdir(grpc_out_dir):
        os.makedirs(grpc_out_dir)

    if not os.path.isdir(csharp_out_dir):
        os.makedirs(csharp_out_dir)

    timestamp = str(datetime.datetime.now())
    for csfile in cs_files_to_generate:
        if csfile.endswith("Grpc.cs"):
            csfile_fullpath = "%s/%s" % (grpc_out_dir, csfile)
        else:
            csfile_fullpath = "%s/%s" % (csharp_out_dir, csfile)
        _write_debug("Creating: %s" % csfile_fullpath)
        with open(csfile_fullpath, "w") as fout:
            print("// Generated by fake protoc: %s" % timestamp, file=fout)


def _create_dependency_file(
    protofile,
    cs_files_to_generate,
    dependencyfile,
    grpc_out_dir,
    csharp_out_dir,
):
    """Create the expected dependency file."""
    _write_debug("\ncreate_dependency_file")

    if not dependencyfile:
        _write_debug("dependencyfile is not set.")
        return

    if not cs_files_to_generate:
        _write_debug("No .cs files matching proto file name %s" % protofile)
        return

    _write_debug("Creating dependency file: %s" % dependencyfile)
    with open(dependencyfile, "w") as out:
        nfiles = len(cs_files_to_generate)
        for i in range(0, nfiles):
            csfile = cs_files_to_generate[i]
            if csfile.endswith("Grpc.cs"):
                cs_filename = os.path.join(grpc_out_dir, csfile)
            else:
                cs_filename = os.path.join(csharp_out_dir, csfile)
            if i == nfiles - 1:
                print("%s: %s" % (cs_filename, protofile), file=out)
            else:
                print("%s \\" % cs_filename, file=out)


def _getenv(name):
    # Note there is a bug in .NET core 3.x that lowercases the environment
    # variable names when they are added via Process.StartInfo, so we need to
    # check both cases here (only an issue on Linux which is case sensitive)
    value = os.getenv(name)
    if value is None:
        value = os.getenv(name.lower())
    return value


def _get_argument_last_occurrence_or_none(protoc_arg_dict, name):
    # If argument was passed multiple times, take the last occurrence.
    # If the value does not exist then return None
    values = protoc_arg_dict.get(name)
    if values is not None:
        return values[-1]
    return None


def main():
    # Check environment variables for the additional arguments used in the tests.

    projectdir = _getenv("FAKEPROTOC_PROJECTDIR")
    if not projectdir:
        print("FAKEPROTOC_PROJECTDIR not set")
        sys.exit(1)
    projectdir = os.path.abspath(projectdir)

    # Output directory for generated files and output file
    protoc_outdir = _getenv("FAKEPROTOC_OUTDIR")
    if not protoc_outdir:
        print("FAKEPROTOC_OUTDIR not set")
        sys.exit(1)
    protoc_outdir = os.path.abspath(protoc_outdir)

    # Get list of expected generated files from env variable
    generate_expected = _getenv("FAKEPROTOC_GENERATE_EXPECTED")
    if not generate_expected:
        print("FAKEPROTOC_GENERATE_EXPECTED not set")
        sys.exit(1)

    # Prepare the debug log
    log_dir = os.path.join(protoc_outdir, "log")
    if not os.path.isdir(log_dir):
        os.makedirs(log_dir)
    _open_debug_log("%s/fakeprotoc_log.txt" % log_dir)

    _write_debug(
        (
            "##### fakeprotoc called at %s\n"
            + "FAKEPROTOC_PROJECTDIR = %s\n"
            + "FAKEPROTOC_GENERATE_EXPECTED = %s\n"
        )
        % (datetime.datetime.now(), projectdir, generate_expected)
    )

    proto_to_generated = _parse_generate_expected(generate_expected)
    protoc_args = _read_protoc_arguments()
    protoc_arg_dict = _parse_protoc_arguments(protoc_args, projectdir)

    # If argument was passed multiple times, take the last occurrence of it.
    # TODO(jtattermusch): handle multiple occurrences of the same argument
    dependencyfile = _get_argument_last_occurrence_or_none(
        protoc_arg_dict, "--dependency_out"
    )
    grpcout = _get_argument_last_occurrence_or_none(
        protoc_arg_dict, "--grpc_out"
    )
    csharpout = _get_argument_last_occurrence_or_none(
        protoc_arg_dict, "--csharp_out"
    )

    # --grpc_out might not be set in which case use --csharp_out
    if grpcout is None:
        grpcout = csharpout

    if len(protoc_arg_dict.get("protofile")) != 1:
        # regular protoc can process multiple .proto files passed at once, but we know
        # the Grpc.Tools msbuild integration only ever passes one .proto file per invocation.
        print(
            "Expecting to get exactly one .proto file argument per fakeprotoc"
            " invocation."
        )
        sys.exit(1)
    protofile = protoc_arg_dict.get("protofile")[0]

    cs_files_to_generate = _get_cs_files_to_generate(
        protofile=protofile, proto_to_generated=proto_to_generated
    )

    _create_dependency_file(
        protofile=protofile,
        cs_files_to_generate=cs_files_to_generate,
        dependencyfile=dependencyfile,
        grpc_out_dir=grpcout,
        csharp_out_dir=csharpout,
    )

    _generate_cs_files(
        protofile=protofile,
        cs_files_to_generate=cs_files_to_generate,
        grpc_out_dir=grpcout,
        csharp_out_dir=csharpout,
        projectdir=projectdir,
    )

    _write_or_update_results_json(
        log_dir=log_dir, protofile=protofile, protoc_arg_dict=protoc_arg_dict
    )

    _close_debug_log()


if __name__ == "__main__":
    main()



# Copyright 2017 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""An example gRPC Python-using server-side application."""

import threading

import grpc

# requests_pb2 is a semantic dependency of this module.
from tests.testing import _application_common
from tests.testing.proto import requests_pb2  # pylint: disable=unused-import
from tests.testing.proto import services_pb2
from tests.testing.proto import services_pb2_grpc


class FirstServiceServicer(services_pb2_grpc.FirstServiceServicer):
    """Services RPCs."""

    def __init__(self):
        self._abort_lock = threading.RLock()
        self._abort_response = _application_common.ABORT_NO_STATUS_RESPONSE

    def UnUn(self, request, context):
        if request == _application_common.UNARY_UNARY_REQUEST:
            return _application_common.UNARY_UNARY_RESPONSE
        elif request == _application_common.ABORT_REQUEST:
            with self._abort_lock:
                try:
                    context.abort(
                        grpc.StatusCode.PERMISSION_DENIED,
                        "Denying permission to test abort.",
                    )
                except Exception as e:  # pylint: disable=broad-except
                    self._abort_response = (
                        _application_common.ABORT_SUCCESS_RESPONSE
                    )
                else:
                    self._abort_status = (
                        _application_common.ABORT_FAILURE_RESPONSE
                    )
            return None  # NOTE: For the linter.
        elif request == _application_common.ABORT_SUCCESS_QUERY:
            with self._abort_lock:
                return self._abort_response
        else:
            context.set_code(grpc.StatusCode.INVALID_ARGUMENT)
            context.set_details("Something is wrong with your request!")
            return services_pb2.Down()

    def UnStre(self, request, context):
        if _application_common.UNARY_STREAM_REQUEST != request:
            context.set_code(grpc.StatusCode.INVALID_ARGUMENT)
            context.set_details("Something is wrong with your request!")
        return
        yield services_pb2.Strange()  # pylint: disable=unreachable

    def StreUn(self, request_iterator, context):
        context.send_initial_metadata(
            (
                (
                    "server_application_metadata_key",
                    "Hi there!",
                ),
            )
        )
        for request in request_iterator:
            if request != _application_common.STREAM_UNARY_REQUEST:
                context.set_code(grpc.StatusCode.INVALID_ARGUMENT)
                context.set_details("Something is wrong with your request!")
                return services_pb2.Strange()
            elif not context.is_active():
                return services_pb2.Strange()
        else:
            return _application_common.STREAM_UNARY_RESPONSE

    def StreStre(self, request_iterator, context):
        valid_requests = (
            _application_common.STREAM_STREAM_REQUEST,
            _application_common.STREAM_STREAM_MUTATING_REQUEST,
        )
        for request in request_iterator:
            if request not in valid_requests:
                context.set_code(grpc.StatusCode.INVALID_ARGUMENT)
                context.set_details("Something is wrong with your request!")
                return
            elif not context.is_active():
                return
            elif request == _application_common.STREAM_STREAM_REQUEST:
                yield _application_common.STREAM_STREAM_RESPONSE
                yield _application_common.STREAM_STREAM_RESPONSE
            elif request == _application_common.STREAM_STREAM_MUTATING_REQUEST:
                response = services_pb2.Bottom()
                for i in range(
                    _application_common.STREAM_STREAM_MUTATING_COUNT
                ):
                    response.first_bottom_field = i
                    yield response



# Copyright 2022 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests of grpc_reflection.v1alpha.reflection."""

import unittest

from google.protobuf.descriptor_pool import DescriptorPool
import grpc
from grpc_reflection.v1alpha import reflection
from grpc_reflection.v1alpha.proto_reflection_descriptor_database import (
    ProtoReflectionDescriptorDatabase,
)

from src.proto.grpc.testing import test_pb2

# Needed to load the EmptyWithExtensions message
from src.proto.grpc.testing.proto2 import empty2_extensions_pb2
from tests.unit import test_common

_PROTO_PACKAGE_NAME = "grpc.testing"
_PROTO_FILE_NAME = "src/proto/grpc/testing/test.proto"
_EMPTY_PROTO_FILE_NAME = "src/proto/grpc/testing/empty.proto"
_INVALID_FILE_NAME = "i-do-not-exist.proto"
_EMPTY_PROTO_SYMBOL_NAME = "grpc.testing.Empty"
_INVALID_SYMBOL_NAME = "IDoNotExist"
_EMPTY_EXTENSIONS_SYMBOL_NAME = "grpc.testing.proto2.EmptyWithExtensions"


class ReflectionClientTest(unittest.TestCase):
    def setUp(self):
        self._server = test_common.test_server()
        self._SERVICE_NAMES = (
            test_pb2.DESCRIPTOR.services_by_name["TestService"].full_name,
            reflection.SERVICE_NAME,
        )
        reflection.enable_server_reflection(self._SERVICE_NAMES, self._server)
        port = self._server.add_insecure_port("[::]:0")
        self._server.start()

        self._channel = grpc.insecure_channel("localhost:%d" % port)

        self._reflection_db = ProtoReflectionDescriptorDatabase(self._channel)
        self.desc_pool = DescriptorPool(self._reflection_db)

    def tearDown(self):
        self._server.stop(None)
        self._channel.close()

    def testListServices(self):
        services = self._reflection_db.get_services()
        self.assertCountEqual(self._SERVICE_NAMES, services)

    def testReflectionServiceName(self):
        self.assertEqual(
            reflection.SERVICE_NAME, "grpc.reflection.v1alpha.ServerReflection"
        )

    def testFindFile(self):
        file_name = _PROTO_FILE_NAME
        file_desc = self.desc_pool.FindFileByName(file_name)
        self.assertEqual(file_name, file_desc.name)
        self.assertEqual(_PROTO_PACKAGE_NAME, file_desc.package)
        self.assertIn("TestService", file_desc.services_by_name)

        file_name = _EMPTY_PROTO_FILE_NAME
        file_desc = self.desc_pool.FindFileByName(file_name)
        self.assertEqual(file_name, file_desc.name)
        self.assertEqual(_PROTO_PACKAGE_NAME, file_desc.package)
        self.assertIn("Empty", file_desc.message_types_by_name)

    def testFindFileError(self):
        with self.assertRaises(KeyError):
            self.desc_pool.FindFileByName(_INVALID_FILE_NAME)

    def testFindMessage(self):
        message_name = _EMPTY_PROTO_SYMBOL_NAME
        message_desc = self.desc_pool.FindMessageTypeByName(message_name)
        self.assertEqual(message_name, message_desc.full_name)
        self.assertTrue(message_name.endswith(message_desc.name))

    def testFindMessageError(self):
        with self.assertRaises(KeyError):
            self.desc_pool.FindMessageTypeByName(_INVALID_SYMBOL_NAME)

    def testFindServiceFindMethod(self):
        service_name = self._SERVICE_NAMES[0]
        service_desc = self.desc_pool.FindServiceByName(service_name)
        self.assertEqual(service_name, service_desc.full_name)
        self.assertTrue(service_name.endswith(service_desc.name))
        file_name = _PROTO_FILE_NAME
        file_desc = self.desc_pool.FindFileByName(file_name)
        self.assertIs(file_desc, service_desc.file)

        method_name = "EmptyCall"
        self.assertIn(method_name, service_desc.methods_by_name)

        method_desc = service_desc.FindMethodByName(method_name)
        self.assertIs(method_desc, service_desc.methods_by_name[method_name])
        self.assertIs(service_desc, method_desc.containing_service)
        self.assertEqual(method_name, method_desc.name)
        self.assertTrue(method_desc.full_name.endswith(method_name))

        empty_message_desc = self.desc_pool.FindMessageTypeByName(
            _EMPTY_PROTO_SYMBOL_NAME
        )
        self.assertEqual(empty_message_desc, method_desc.input_type)
        self.assertEqual(empty_message_desc, method_desc.output_type)

    def testFindServiceError(self):
        with self.assertRaises(KeyError):
            self.desc_pool.FindServiceByName(_INVALID_SYMBOL_NAME)

    def testFindMethodError(self):
        service_name = self._SERVICE_NAMES[0]
        service_desc = self.desc_pool.FindServiceByName(service_name)

        # FindMethodByName sometimes raises a KeyError, and sometimes returns None.
        # See https://github.com/protocolbuffers/protobuf/issues/9592
        with self.assertRaises(KeyError):
            res = service_desc.FindMethodByName(_INVALID_SYMBOL_NAME)
            if res is None:
                raise KeyError()

    def testFindExtensionNotImplemented(self):
        """
        Extensions aren't implemented in Protobuf for Python.
        For now, simply assert that indeed they don't work.
        """
        message_name = _EMPTY_EXTENSIONS_SYMBOL_NAME
        message_desc = self.desc_pool.FindMessageTypeByName(message_name)
        self.assertEqual(message_name, message_desc.full_name)
        self.assertTrue(message_name.endswith(message_desc.name))
        extension_field_descs = self.desc_pool.FindAllExtensions(message_desc)

        self.assertEqual(0, len(extension_field_descs))
        with self.assertRaises(KeyError):
            self.desc_pool.FindExtensionByName(message_name)


if __name__ == "__main__":
    unittest.main(verbosity=2)



# Copyright 2019 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Client-side fork interop tests as a unit test."""

import os
import subprocess
import sys
import tempfile
import threading
import time
import unittest

from grpc._cython import cygrpc

from tests.fork import debugger
from tests.fork import methods


def _dump_streams(name, streams):
    assert len(streams) == 2
    for stream_name, stream in zip(("STDOUT", "STDERR"), streams):
        stream.seek(0)
        sys.stderr.write(
            "{} {}:\n{}\n".format(name, stream_name, stream.read().decode())
        )
        stream.close()
    sys.stderr.flush()


# New instance of multiprocessing.Process using fork without exec can and will
# freeze if the Python process has any other threads running. This includes the
# additional thread spawned by our _runner.py class. So in order to test our
# compatibility with multiprocessing, we first fork+exec a new process to ensure
# we don't have any conflicting background threads.
_CLIENT_FORK_SCRIPT_TEMPLATE = """if True:
    import os
    from grpc._cython import cygrpc
    from tests.fork import methods

    from src.python.grpcio_tests.tests.fork import native_debug

    native_debug.install_failure_signal_handler()
    methods.setup_logger()

    cygrpc._GRPC_ENABLE_FORK_SUPPORT = True
    os.environ['GRPC_ENABLE_FORK_SUPPORT'] = 'true'
    methods.TestCase.%s.run_test({
      'server_host': 'localhost',
      'server_port': %d,
      'use_tls': False
    })
"""
_SUBPROCESS_TIMEOUT_S = 80


@unittest.skipUnless(
    sys.platform.startswith("linux") or sys.platform.startswith("darwin"),
    f"not supported on {sys.platform}",
)
@unittest.skipUnless(
    os.getenv("GRPC_ENABLE_FORK_SUPPORT") is not None,
    "Core must be built with fork support to run this test.",
)
class ForkInteropTest(unittest.TestCase):
    def setUp(self):
        self._port = None
        start_server_script = """if True:
            import sys
            import time

            import grpc
            from src.proto.grpc.testing import test_pb2_grpc
            from tests.interop import service as interop_service
            from tests.unit import test_common

            server = test_common.test_server()
            test_pb2_grpc.add_TestServiceServicer_to_server(
                interop_service.TestService(), server)
            port = server.add_insecure_port('[::]:0')
            server.start()
            print(port)
            sys.stdout.flush()
            while True:
                time.sleep(1)
        """
        self._streams = tuple(tempfile.TemporaryFile() for _ in range(2))
        self._server_process = subprocess.Popen(
            [sys.executable, "-c", start_server_script],
            stdout=self._streams[0],
            stderr=self._streams[1],
        )
        timer = threading.Timer(
            _SUBPROCESS_TIMEOUT_S, self._server_process.kill
        )
        interval_secs = 2.0
        cumulative_secs = 0.0
        try:
            timer.start()
            while cumulative_secs < _SUBPROCESS_TIMEOUT_S:
                self._streams[0].seek(0)
                s = self._streams[0].readline()
                if s:
                    self._port = int(s)
                    break
                time.sleep(interval_secs)
                cumulative_secs += interval_secs

            if self._port is None:
                # Timeout
                self._streams[0].seek(0)
                sys.stderr.write(
                    "Server STDOUT:\n{}\n".format(self._streams[0].read())
                )
                self._streams[1].seek(0)
                sys.stderr.write(
                    "Server STDERR:\n{}\n".format(self._streams[1].read())
                )
                sys.stderr.flush()
                raise Exception("Failed to get port from server.")
        except ValueError:
            raise Exception("Failed to get port from server")
        finally:
            timer.cancel()

    def testConnectivityWatch(self):
        self._verifyTestCase(methods.TestCase.CONNECTIVITY_WATCH)

    def testCloseChannelBeforeFork(self):
        self._verifyTestCase(methods.TestCase.CLOSE_CHANNEL_BEFORE_FORK)

    def testAsyncUnarySameChannel(self):
        self._verifyTestCase(methods.TestCase.ASYNC_UNARY_SAME_CHANNEL)

    def testAsyncUnaryNewChannel(self):
        self._verifyTestCase(methods.TestCase.ASYNC_UNARY_NEW_CHANNEL)

    def testBlockingUnarySameChannel(self):
        self._verifyTestCase(methods.TestCase.BLOCKING_UNARY_SAME_CHANNEL)

    def testBlockingUnaryNewChannel(self):
        self._verifyTestCase(methods.TestCase.BLOCKING_UNARY_NEW_CHANNEL)

    def testInProgressBidiContinueCall(self):
        self._verifyTestCase(methods.TestCase.IN_PROGRESS_BIDI_CONTINUE_CALL)

    def testInProgressBidiSameChannelAsyncCall(self):
        self._verifyTestCase(
            methods.TestCase.IN_PROGRESS_BIDI_SAME_CHANNEL_ASYNC_CALL
        )

    def testInProgressBidiSameChannelBlockingCall(self):
        self._verifyTestCase(
            methods.TestCase.IN_PROGRESS_BIDI_SAME_CHANNEL_BLOCKING_CALL
        )

    def testInProgressBidiNewChannelAsyncCall(self):
        self._verifyTestCase(
            methods.TestCase.IN_PROGRESS_BIDI_NEW_CHANNEL_ASYNC_CALL
        )

    def testInProgressBidiNewChannelBlockingCall(self):
        self._verifyTestCase(
            methods.TestCase.IN_PROGRESS_BIDI_NEW_CHANNEL_BLOCKING_CALL
        )

    def tearDown(self):
        self._server_process.kill()
        for stream in self._streams:
            stream.close()

    def _verifyTestCase(self, test_case):
        script = _CLIENT_FORK_SCRIPT_TEMPLATE % (test_case.name, self._port)
        streams = tuple(tempfile.TemporaryFile() for _ in range(2))
        process = subprocess.Popen(
            [sys.executable, "-c", script], stdout=streams[0], stderr=streams[1]
        )
        try:
            process.wait(timeout=_SUBPROCESS_TIMEOUT_S)
            self.assertEqual(0, process.returncode)
        except subprocess.TimeoutExpired:
            sys.stderr.write("Child %d timed out\n" % process.pid)
            debugger.print_backtraces(process.pid)
            process.kill()
            raise AssertionError("Parent process timed out.")
        finally:
            _dump_streams("Parent", streams)
            _dump_streams("Server", self._streams)


if __name__ == "__main__":
    unittest.main(verbosity=2)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/type/matcher/v3/address.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/type/matcher/v3/address.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from xds.core.v3 import cidr_pb2 as xds_dot_core_dot_v3_dot_cidr__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n#envoy/type/matcher/v3/address.proto\x12\x15\x65nvoy.type.matcher.v3\x1a\x16xds/core/v3/cidr.proto\x1a\x1dudpa/annotations/status.proto\"8\n\x0e\x41\x64\x64ressMatcher\x12&\n\x06ranges\x18\x01 \x03(\x0b\x32\x16.xds.core.v3.CidrRangeB\x85\x01\n#io.envoyproxy.envoy.type.matcher.v3B\x0c\x41\x64\x64ressProtoP\x01ZFgithub.com/envoyproxy/go-control-plane/envoy/type/matcher/v3;matcherv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.type.matcher.v3.address_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n#io.envoyproxy.envoy.type.matcher.v3B\014AddressProtoP\001ZFgithub.com/envoyproxy/go-control-plane/envoy/type/matcher/v3;matcherv3\272\200\310\321\006\002\020\002'
  _globals['_ADDRESSMATCHER']._serialized_start=117
  _globals['_ADDRESSMATCHER']._serialized_end=173
# @@protoc_insertion_point(module_scope)



# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Secure client-server interoperability as a unit test."""

import unittest

import grpc
import grpc.experimental

from src.proto.grpc.testing import test_pb2_grpc
from tests.interop import _intraop_test_case
from tests.interop import resources
from tests.interop import service
from tests.unit import test_common

_SERVER_HOST_OVERRIDE = "foo.test.google.fr"


class SecureIntraopTest(_intraop_test_case.IntraopTestCase, unittest.TestCase):
    def setUp(self):
        self.server = test_common.test_server()
        test_pb2_grpc.add_TestServiceServicer_to_server(
            service.TestService(), self.server
        )
        port = self.server.add_secure_port(
            "[::]:0",
            grpc.ssl_server_credentials(
                [
                    (
                        resources.server_private_key(),
                        resources.certificate_chain(),
                    )
                ]
            ),
        )
        self.server.start()
        self.channel = grpc.secure_channel(
            "localhost:{}".format(port),
            grpc.ssl_channel_credentials(resources.test_root_certificates()),
            (
                (
                    "grpc.ssl_target_name_override",
                    _SERVER_HOST_OVERRIDE,
                ),
            ),
        )
        self.stub = test_pb2_grpc.TestServiceStub(self.channel)

    def tearDown(self):
        self.server.stop(None)
        self.channel.close()


class SecureInteropWithSyncPrivateKeyOffloadingTest(
    _intraop_test_case.IntraopTestCase, unittest.TestCase
):

    def setUp(self):
        self.server = test_common.test_server()
        test_pb2_grpc.add_TestServiceServicer_to_server(
            service.TestService(), self.server
        )
        # Configure the server for mTLS so the client will do Private Key signing
        port = self.server.add_secure_port(
            "[::]:0",
            grpc.ssl_server_credentials(
                [
                    (
                        resources.server_private_key(),
                        resources.certificate_chain(),
                    )
                ],
                resources.test_root_certificates(),
                require_client_auth=True,
            ),
        )
        self.server.start()
        self.channel = grpc.secure_channel(
            "localhost:{}".format(port),
            grpc.experimental.ssl_channel_credentials_with_custom_signer(
                private_key_sign_fn=resources.sync_client_private_key_signer,
                root_certificates=resources.test_root_certificates(),
                certificate_chain=resources.client_certificate_chain(),
            ),
            (
                (
                    "grpc.ssl_target_name_override",
                    _SERVER_HOST_OVERRIDE,
                ),
            ),
        )
        self.stub = test_pb2_grpc.TestServiceStub(self.channel)

    def tearDown(self):
        self.server.stop(None)
        self.channel.close()


class SecureInteropWithAsyncPrivateKeyOffloadingTest(
    _intraop_test_case.IntraopTestCase, unittest.TestCase
):

    def setUp(self):
        self.server = test_common.test_server()
        test_pb2_grpc.add_TestServiceServicer_to_server(
            service.TestService(), self.server
        )
        # Configure the server for mTLS so the client will do Private Key signing
        port = self.server.add_secure_port(
            "[::]:0",
            grpc.ssl_server_credentials(
                [
                    (
                        resources.server_private_key(),
                        resources.certificate_chain(),
                    )
                ],
                resources.test_root_certificates(),
                require_client_auth=True,
            ),
        )
        self.server.start()
        self.channel = grpc.secure_channel(
            "localhost:{}".format(port),
            grpc.experimental.ssl_channel_credentials_with_custom_signer(
                private_key_sign_fn=resources.async_client_private_key_signer,
                root_certificates=resources.test_root_certificates(),
                certificate_chain=resources.client_certificate_chain(),
            ),
            (
                (
                    "grpc.ssl_target_name_override",
                    _SERVER_HOST_OVERRIDE,
                ),
            ),
        )
        self.stub = test_pb2_grpc.TestServiceStub(self.channel)

    def tearDown(self):
        self.server.stop(None)
        self.channel.close()


if __name__ == "__main__":
    unittest.main(verbosity=2)
